/*
 *  upkg: tool for manipulating Unreal Tournament packages.
 *  Copyright (C) 2009 Nick Bowler
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <string.h>
#include <glib-object.h>

#include <uobject/uobject.h>
#include <uobject/exportable.h>
#include "texture.h"
#include "upkg.h"

#define TEXTURE_GET_PRIV(o) \
	G_TYPE_INSTANCE_GET_PRIVATE(o, ENGINE_TEXTURE_TYPE, struct texture_priv)

enum {
	PROP_0,
	PROP_USIZE,
	PROP_VSIZE,
};

struct texture_priv {
	unsigned type;

	struct upkg_file *f;
	size_t base, len;

	unsigned char buf[2048];
	unsigned long nbuf;
};

void exportable_init(UObjectExportable *e)
{
}

G_DEFINE_DYNAMIC_TYPE(EngineTexture, engine_texture, U_OBJECT_TYPE);

static int deserialize(UObject *o, struct upkg_file *f)
{
	struct texture_priv *priv = TEXTURE_GET_PRIV(o);
	EngineTexture *t = ENGINE_TEXTURE(o);

	U_OBJECT_CLASS(engine_texture_parent_class)->deserialize(o, f);

	printf("Texture size: %ux%u\n", t->usize, t->vsize);

	return 0;
}

void texture_register(GTypeModule *m)
{
	engine_texture_register_type(m);
}

static void
set_property(GObject *o, guint id, const GValue *val, GParamSpec *spec)
{
	EngineTexture *t = ENGINE_TEXTURE(o);

	switch (id) {
	case PROP_USIZE:
		t->usize = g_value_get_uint(val);
		break;
	case PROP_VSIZE:
		t->vsize = g_value_get_uint(val);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID(o, id, spec);
	}
}

static void
get_property(GObject *o, guint id, GValue *val, GParamSpec *spec)
{
	EngineTexture *t = ENGINE_TEXTURE(o);

	switch (id) {
	case PROP_USIZE:
		g_value_set_boolean(val, t->usize);
		break;
	case PROP_VSIZE:
		g_value_set_boolean(val, t->vsize);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID(o, id, spec);
	}
}

static void engine_texture_init(EngineTexture *t)
{
	t->usize = t->vsize = 0;
}

static void engine_texture_class_init(EngineTextureClass *class)
{
	UObjectClass *uo = U_OBJECT_CLASS(class);
	GObjectClass *go = G_OBJECT_CLASS(class);

	g_type_class_add_private(class, sizeof (struct texture_priv));

	uo->deserialize = deserialize;
	go->set_property = set_property;
	go->get_property = get_property;

	g_object_class_install_property(go, PROP_USIZE,
		g_param_spec_uint("USize",
			"USize",
			"Width of the texture.",
			0, 2048, 0,
			G_PARAM_READWRITE));
	g_object_class_install_property(go, PROP_VSIZE,
		g_param_spec_uint("VSize",
			"VSize",
			"Height of the texture.",
			0, 2048, 0,
			G_PARAM_READWRITE));
}

static void engine_texture_class_finalize(EngineTextureClass *class)
{
}
